@@ -9,6 +9,6 @@ path = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..')
 for a, b in dirs:
      rst = [os.path.splitext(x)[0].lower() for x in os.listdir(os.path.join(path, 'documentation', 'source', a))]
      py = [os.path.splitext(x)[0].lower() for x in os.listdir(os.path.join(path, b))]
-     print a
+     print(a)
      for x in set(py) - set(rst):
-         print "    ", x
+         print( "    ", x)
