@@ -2,15 +2,13 @@
 """
 In this example we draw two different kinds of histogram.
 """
-
-
 import initExample ## Add path to library (just for examples; you do not need this)
 
 import pyqtgraph as pg
 from pyqtgraph.Qt import QtCore, QtGui
 import numpy as np
 
-win = pg.GraphicsWindow()
+win = pg.GraphicsLayoutWidget(show=True)
 win.resize(800,350)
 win.setWindowTitle('pyqtgraph example: Histogram')
 plt1 = win.addPlot()
@@ -22,11 +20,9 @@ vals = np.hstack([np.random.normal(size=500), np.random.normal(size=260, loc=4)]
 ## compute standard histogram
 y,x = np.histogram(vals, bins=np.linspace(-3, 8, 40))
 
+## Using stepMode="center" causes the plot to draw two lines for each sample.
 ## notice that len(x) == len(y)+1
-## We are required to use stepMode=True so that PlotCurveItem will interpret this data correctly.
-curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=(0, 0, 255, 80))
-plt1.addItem(curve)
-
+plt1.plot(x, y, stepMode="center", fillLevel=0, fillOutline=True, brush=(0,0,255,150))
 
 ## Now draw all points as a nicely-spaced scatter plot
 y = pg.pseudoScatter(vals, spacing=0.15)
