@@ -1,13 +1,17 @@
+# -*- coding: utf-8 -*-
 import sys, re, os, time, traceback, subprocess
 import pickle
 
-from ..Qt import QtCore, QtGui, USE_PYSIDE, USE_PYQT5
+from ..Qt import QtCore, QtGui, QT_LIB
 from ..python2_3 import basestring
 from .. import exceptionHandling as exceptionHandling
 from .. import getConfigOption
-if USE_PYSIDE:
+from ..functions import SignalBlock
+if QT_LIB == 'PySide':
     from . import template_pyside as template
-elif USE_PYQT5:
+elif QT_LIB == 'PySide2':
+    from . import template_pyside2 as template
+elif QT_LIB == 'PyQt5':
     from . import template_pyqt5 as template
 else:
     from . import template_pyqt as template
@@ -31,6 +35,7 @@ class ConsoleWidget(QtGui.QWidget):
     - ability to add extra features like exception stack introspection
     - ability to have multiple interactive prompts, including for spawned sub-processes
     """
+    _threadException = QtCore.Signal(object)
     
     def __init__(self, parent=None, namespace=None, historyFile=None, text=None, editor=None):
         """
@@ -48,10 +53,12 @@ class ConsoleWidget(QtGui.QWidget):
         QtGui.QWidget.__init__(self, parent)
         if namespace is None:
             namespace = {}
+        namespace['__console__'] = self
         self.localNamespace = namespace
         self.editor = editor
         self.multiline = None
         self.inCmd = False
+        self.frames = []  # stack frames to access when an item in the stack list is selected
         
         self.ui = template.Ui_Form()
         self.ui.setupUi(self)
@@ -85,21 +92,27 @@ class ConsoleWidget(QtGui.QWidget):
         self.ui.onlyUncaughtCheck.toggled.connect(self.updateSysTrace)
         
         self.currentTraceback = None
+
+        # send exceptions raised in non-gui threads back to the main thread by signal.
+        self._threadException.connect(self._threadExceptionHandler)
         
     def loadHistory(self):
         """Return the list of previously-invoked command strings (or None)."""
         if self.historyFile is not None:
-            return pickle.load(open(self.historyFile, 'rb'))
+            with open(self.historyFile, 'rb') as pf:
+                return pickle.load(pf)
         
     def saveHistory(self, history):
         """Store the list of previously-invoked command strings."""
         if self.historyFile is not None:
-            pickle.dump(open(self.historyFile, 'wb'), history)
+            with open(self.historyFile, 'wb') as pf:
+                pickle.dump(pf, history)
         
     def runCmd(self, cmd):
         #cmd = str(self.input.lastCmd)
-        self.stdout = sys.stdout
-        self.stderr = sys.stderr
+
+        orig_stdout = sys.stdout
+        orig_stderr = sys.stderr
         encCmd = re.sub(r'>', '&gt;', re.sub(r'<', '&lt;', cmd))
         encCmd = re.sub(r' ', '&nbsp;', encCmd)
         
@@ -110,69 +123,68 @@ class ConsoleWidget(QtGui.QWidget):
             sys.stdout = self
             sys.stderr = self
             if self.multiline is not None:
-                self.write("<br><b>%s</b>\n"%encCmd, html=True)
+                self.write("<br><b>%s</b>\n"%encCmd, html=True, scrollToBottom=True)
                 self.execMulti(cmd)
             else:
-                self.write("<br><div style='background-color: #CCF'><b>%s</b>\n"%encCmd, html=True)
+                self.write("<br><div style='background-color: #CCF; color: black'><b>%s</b>\n"%encCmd, html=True, scrollToBottom=True)
                 self.inCmd = True
                 self.execSingle(cmd)
             
             if not self.inCmd:
-                self.write("</div>\n", html=True)
+                self.write("</div>\n", html=True, scrollToBottom=True)
                 
         finally:
-            sys.stdout = self.stdout
-            sys.stderr = self.stderr
+            sys.stdout = orig_stdout
+            sys.stderr = orig_stderr
             
-            sb = self.output.verticalScrollBar()
-            sb.setValue(sb.maximum())
             sb = self.ui.historyList.verticalScrollBar()
             sb.setValue(sb.maximum())
             
     def globals(self):
         frame = self.currentFrame()
         if frame is not None and self.ui.runSelectedFrameCheck.isChecked():
-            return self.currentFrame().tb_frame.f_globals
+            return self.currentFrame().f_globals
         else:
-            return globals()
+            return self.localNamespace
         
     def locals(self):
         frame = self.currentFrame()
         if frame is not None and self.ui.runSelectedFrameCheck.isChecked():
-            return self.currentFrame().tb_frame.f_locals
+            return self.currentFrame().f_locals
         else:
             return self.localNamespace
             
     def currentFrame(self):
         ## Return the currently selected exception stack frame (or None if there is no exception)
-        if self.currentTraceback is None:
-            return None
         index = self.ui.exceptionStackList.currentRow()
-        tb = self.currentTraceback
-        for i in range(index):
-            tb = tb.tb_next
-        return tb
+        if index >= 0 and index < len(self.frames):
+            return self.frames[index]
+        else:
+            return None
         
     def execSingle(self, cmd):
         try:
             output = eval(cmd, self.globals(), self.locals())
             self.write(repr(output) + '\n')
+            return
         except SyntaxError:
-            try:
-                exec(cmd, self.globals(), self.locals())
-            except SyntaxError as exc:
-                if 'unexpected EOF' in exc.msg:
-                    self.multiline = cmd
-                else:
-                    self.displayException()
-            except:
+            pass
+        except:
+            self.displayException()
+            return
+
+        # eval failed with syntax error; try exec instead
+        try:
+            exec(cmd, self.globals(), self.locals())
+        except SyntaxError as exc:
+            if 'unexpected EOF' in exc.msg:
+                self.multiline = cmd
+            else:
                 self.displayException()
         except:
             self.displayException()
             
-            
     def execMulti(self, nextLine):
-        #self.stdout.write(nextLine+"\n")
         if nextLine.strip() != '':
             self.multiline += "\n" + nextLine
             return
@@ -183,35 +195,65 @@ class ConsoleWidget(QtGui.QWidget):
             output = eval(cmd, self.globals(), self.locals())
             self.write(str(output) + '\n')
             self.multiline = None
+            return
         except SyntaxError:
-            try:
-                exec(cmd, self.globals(), self.locals())
-                self.multiline = None
-            except SyntaxError as exc:
-                if 'unexpected EOF' in exc.msg:
-                    self.multiline = cmd
-                else:
-                    self.displayException()
-                    self.multiline = None
-            except:
+            pass
+        except:
+            self.displayException()
+            self.multiline = None
+            return
+
+        # eval failed with syntax error; try exec instead
+        try:
+            exec(cmd, self.globals(), self.locals())
+            self.multiline = None
+        except SyntaxError as exc:
+            if 'unexpected EOF' in exc.msg:
+                self.multiline = cmd
+            else:
                 self.displayException()
                 self.multiline = None
         except:
             self.displayException()
             self.multiline = None
 
-    def write(self, strn, html=False):
+
+    def write(self, strn, html=False, scrollToBottom='auto'):
+        """Write a string into the console.
+
+        If scrollToBottom is 'auto', then the console is automatically scrolled
+        to fit the new text only if it was already at the bottom.
+        """
+        isGuiThread = QtCore.QThread.currentThread() == QtCore.QCoreApplication.instance().thread()
+        if not isGuiThread:
+            sys.__stdout__.write(strn)
+            return
+
+        sb = self.output.verticalScrollBar()
+        scroll = sb.value()
+        if scrollToBottom == 'auto':
+            atBottom = scroll == sb.maximum()
+            scrollToBottom = atBottom
+
         self.output.moveCursor(QtGui.QTextCursor.End)
         if html:
             self.output.textCursor().insertHtml(strn)
         else:
             if self.inCmd:
                 self.inCmd = False
-                self.output.textCursor().insertHtml("</div><br><div style='font-weight: normal; background-color: #FFF;'>")
-                #self.stdout.write("</div><br><div style='font-weight: normal; background-color: #FFF;'>")
+                self.output.textCursor().insertHtml("</div><br><div style='font-weight: normal; background-color: #FFF; color: black'>")
             self.output.insertPlainText(strn)
-        #self.stdout.write(strn)
+
+        if scrollToBottom:
+            sb.setValue(sb.maximum())
+        else:
+            sb.setValue(scroll)
+
     
+    def fileno(self):
+        # Need to implement this since we temporarily occlude sys.stdout, and someone may be looking for it (faulthandler, for example)
+        return 1
+
     def displayException(self):
         """
         Display the current exception and stack.
@@ -243,9 +285,12 @@ class ConsoleWidget(QtGui.QWidget):
         If True, the console will catch all unhandled exceptions and display the stack
         trace. Each exception caught clears the last.
         """
-        self.ui.catchAllExceptionsBtn.setChecked(catch)
+        with SignalBlock(self.ui.catchAllExceptionsBtn.toggled, self.catchAllExceptions):
+            self.ui.catchAllExceptionsBtn.setChecked(catch)
+        
         if catch:
-            self.ui.catchNextExceptionBtn.setChecked(False)
+            with SignalBlock(self.ui.catchNextExceptionBtn.toggled, self.catchNextException):
+                self.ui.catchNextExceptionBtn.setChecked(False)
             self.enableExceptionHandling()
             self.ui.exceptionBtn.setChecked(True)
         else:
@@ -256,9 +301,11 @@ class ConsoleWidget(QtGui.QWidget):
         If True, the console will catch the next unhandled exception and display the stack
         trace.
         """
-        self.ui.catchNextExceptionBtn.setChecked(catch)
+        with SignalBlock(self.ui.catchNextExceptionBtn.toggled, self.catchNextException):
+            self.ui.catchNextExceptionBtn.setChecked(catch)
         if catch:
-            self.ui.catchAllExceptionsBtn.setChecked(False)
+            with SignalBlock(self.ui.catchAllExceptionsBtn.toggled, self.catchAllExceptions):
+                self.ui.catchAllExceptionsBtn.setChecked(False)
             self.enableExceptionHandling()
             self.ui.exceptionBtn.setChecked(True)
         else:
@@ -274,6 +321,7 @@ class ConsoleWidget(QtGui.QWidget):
         
     def clearExceptionClicked(self):
         self.currentTraceback = None
+        self.frames = []
         self.ui.exceptionInfoLabel.setText("[No current exception]")
         self.ui.exceptionStackList.clear()
         self.ui.clearExceptionBtn.setEnabled(False)
@@ -288,18 +336,10 @@ class ConsoleWidget(QtGui.QWidget):
         if editor is None:
             return
         tb = self.currentFrame()
-        lineNum = tb.tb_lineno
-        fileName = tb.tb_frame.f_code.co_filename
+        lineNum = tb.f_lineno
+        fileName = tb.f_code.co_filename
         subprocess.Popen(self.editor.format(fileName=fileName, lineNum=lineNum), shell=True)
         
-    
-    #def allExceptionsHandler(self, *args):
-        #self.exceptionHandler(*args)
-    
-    #def nextExceptionHandler(self, *args):
-        #self.ui.catchNextExceptionBtn.setChecked(False)
-        #self.exceptionHandler(*args)
-
     def updateSysTrace(self):
         ## Install or uninstall  sys.settrace handler 
         
@@ -318,24 +358,95 @@ class ConsoleWidget(QtGui.QWidget):
             else:
                 sys.settrace(self.systrace)
         
-    def exceptionHandler(self, excType, exc, tb):
+    def exceptionHandler(self, excType, exc, tb, systrace=False, frame=None):
+        if frame is None:
+            frame = sys._getframe()
+
+        # exceptions raised in non-gui threads must be handled separately
+        isGuiThread = QtCore.QThread.currentThread() == QtCore.QCoreApplication.instance().thread()
+        if not isGuiThread:
+            # sending a frame from one thread to another.. probably not safe, but better than just
+            # dropping the exception?
+            self._threadException.emit((excType, exc, tb, systrace, frame.f_back))
+            return
+
         if self.ui.catchNextExceptionBtn.isChecked():
             self.ui.catchNextExceptionBtn.setChecked(False)
         elif not self.ui.catchAllExceptionsBtn.isChecked():
             return
         
-        self.ui.clearExceptionBtn.setEnabled(True)
         self.currentTraceback = tb
         
         excMessage = ''.join(traceback.format_exception_only(excType, exc))
         self.ui.exceptionInfoLabel.setText(excMessage)
-        self.ui.exceptionStackList.clear()
-        for index, line in enumerate(traceback.extract_tb(tb)):
-            self.ui.exceptionStackList.addItem('File "%s", line %s, in %s()\n  %s' % line)
+
+        if systrace:
+            # exceptions caught using systrace don't need the usual 
+            # call stack + traceback handling
+            self.setStack(frame.f_back.f_back)
+        else:
+            self.setStack(frame=frame.f_back, tb=tb)
     
+    def _threadExceptionHandler(self, args):
+        self.exceptionHandler(*args)
+
+    def setStack(self, frame=None, tb=None):
+        """Display a call stack and exception traceback.
+
+        This allows the user to probe the contents of any frame in the given stack.
+
+        *frame* may either be a Frame instance or None, in which case the current 
+        frame is retrieved from ``sys._getframe()``. 
+
+        If *tb* is provided then the frames in the traceback will be appended to 
+        the end of the stack list. If *tb* is None, then sys.exc_info() will 
+        be checked instead.
+        """
+        self.ui.clearExceptionBtn.setEnabled(True)
+        
+        if frame is None:
+            frame = sys._getframe().f_back
+
+        if tb is None:
+            tb = sys.exc_info()[2]
+
+        self.ui.exceptionStackList.clear()
+        self.frames = []
+
+        # Build stack up to this point
+        for index, line in enumerate(traceback.extract_stack(frame)):
+            # extract_stack return value changed in python 3.5
+            if 'FrameSummary' in str(type(line)):
+                line = (line.filename, line.lineno, line.name, line._line)
+            
+            self.ui.exceptionStackList.addItem('File "%s", line %s, in %s()\n  %s' % line)
+        while frame is not None:
+            self.frames.insert(0, frame)
+            frame = frame.f_back
+
+        if tb is None:
+            return
+
+        self.ui.exceptionStackList.addItem('-- exception caught here: --')
+        item = self.ui.exceptionStackList.item(self.ui.exceptionStackList.count()-1)
+        item.setBackground(QtGui.QBrush(QtGui.QColor(200, 200, 200)))
+        item.setForeground(QtGui.QBrush(QtGui.QColor(50, 50, 50)))
+        self.frames.append(None)
+
+        # And finish the rest of the stack up to the exception
+        for index, line in enumerate(traceback.extract_tb(tb)):
+            # extract_stack return value changed in python 3.5
+            if 'FrameSummary' in str(type(line)):
+                line = (line.filename, line.lineno, line.name, line._line)
+            
+            self.ui.exceptionStackList.addItem('File "%s", line %s, in %s()\n  %s' % line)
+        while tb is not None:
+            self.frames.append(tb.tb_frame)
+            tb = tb.tb_next
+
     def systrace(self, frame, event, arg):
         if event == 'exception' and self.checkException(*arg):
-            self.exceptionHandler(*arg)
+            self.exceptionHandler(*arg, systrace=True)
         return self.systrace
         
     def checkException(self, excType, exc, tb):
@@ -347,7 +458,7 @@ class ConsoleWidget(QtGui.QWidget):
         filterStr = str(self.ui.filterText.text())
         if filterStr != '':
             if isinstance(exc, Exception):
-                msg = exc.message
+                msg = traceback.format_exception_only(type(exc), exc)
             elif isinstance(exc, basestring):
                 msg = exc
             else:
