@@ -1,14 +1,20 @@
 # -*- coding: utf-8 -*-
-from ..Qt import QtCore, QtGui, USE_PYSIDE
+from ..Qt import QtCore, QtGui, QT_LIB
 from .Node import *
 from ..pgcollections import OrderedDict
 from ..widgets.TreeWidget import *
 from .. import FileDialog, DataTreeWidget
 
 ## pyside and pyqt use incompatible ui files.
-if USE_PYSIDE:
+if QT_LIB == 'PySide':
     from . import FlowchartTemplate_pyside as FlowchartTemplate
     from . import FlowchartCtrlTemplate_pyside as FlowchartCtrlTemplate
+elif QT_LIB == 'PySide2':
+    from . import FlowchartTemplate_pyside2 as FlowchartTemplate
+    from . import FlowchartCtrlTemplate_pyside2 as FlowchartCtrlTemplate
+elif QT_LIB == 'PyQt5':
+    from . import FlowchartTemplate_pyqt5 as FlowchartTemplate
+    from . import FlowchartCtrlTemplate_pyqt5 as FlowchartCtrlTemplate
 else:
     from . import FlowchartTemplate_pyqt as FlowchartTemplate
     from . import FlowchartCtrlTemplate_pyqt as FlowchartCtrlTemplate
@@ -21,6 +27,7 @@ from .. import configfile as configfile
 from .. import dockarea as dockarea
 from . import FlowchartGraphicsView
 from .. import functions as fn
+from ..python2_3 import asUnicode
 
 def strDict(d):
     return dict([(str(k), v) for k, v in d.items()])
@@ -163,6 +170,8 @@ class Flowchart(Node):
                 n[oldName].rename(newName)
 
     def createNode(self, nodeType, name=None, pos=None):
+        """Create a new Node and add it to this flowchart.
+        """
         if name is None:
             n = 0
             while True:
@@ -176,6 +185,10 @@ class Flowchart(Node):
         return node
         
     def addNode(self, node, name, pos=None):
+        """Add an existing Node to this flowchart.
+        
+        See also: createNode()
+        """
         if pos is None:
             pos = [0, 0]
         if type(pos) in [QtCore.QPoint, QtCore.QPointF]:
@@ -186,21 +199,26 @@ class Flowchart(Node):
         self.viewBox.addItem(item)
         item.moveBy(*pos)
         self._nodes[name] = node
-        self.widget().addNode(node) 
+        if node is not self.inputNode and node is not self.outputNode:
+            self.widget().addNode(node) 
         node.sigClosed.connect(self.nodeClosed)
         node.sigRenamed.connect(self.nodeRenamed)
         node.sigOutputChanged.connect(self.nodeOutputChanged)
         self.sigChartChanged.emit(self, 'add', node)
         
     def removeNode(self, node):
+        """Remove a Node from this flowchart.
+        """
         node.close()
         
     def nodeClosed(self, node):
         del self._nodes[node.name()]
         self.widget().removeNode(node)
-        for signal in ['sigClosed', 'sigRenamed', 'sigOutputChanged']:
+        for signal, slot in [('sigClosed', self.nodeClosed),
+                             ('sigRenamed', self.nodeRenamed),
+                             ('sigOutputChanged', self.nodeOutputChanged)]:
             try:
-                getattr(node, signal).disconnect(self.nodeClosed)
+                getattr(node, signal).disconnect(slot)
             except (TypeError, RuntimeError):
                 pass
         self.sigChartChanged.emit(self, 'remove', node)
@@ -230,7 +248,6 @@ class Flowchart(Node):
         term2 = self.internalTerminal(term2)
         term1.connectTo(term2)
         
-        
     def process(self, **args):
         """
         Process data through the flowchart, returning the output.
@@ -322,7 +339,6 @@ class Flowchart(Node):
             
         #print "DEPS:", deps
         ## determine correct node-processing order
-        #deps[self] = []
         order = fn.toposort(deps)
         #print "ORDER1:", order
         
@@ -346,10 +362,8 @@ class Flowchart(Node):
                 if lastNode is None or ind > lastInd:
                     lastNode = n
                     lastInd = ind
-            #tdeps[t] = lastNode
             if lastInd is not None:
                 dels.append((lastInd+1, t))
-        #dels.sort(lambda a,b: cmp(b[0], a[0]))
         dels.sort(key=lambda a: a[0], reverse=True)
         for i, t in dels:
             ops.insert(i, ('d', t))
@@ -379,22 +393,22 @@ class Flowchart(Node):
             terms = set(startNode.outputs().values())
             
             #print "======= Updating", startNode
-            #print "Order:", order
+            # print("Order:", order)
             for node in order[1:]:
-                #print "Processing node", node
+                # print("Processing node", node)
+                update = False
                 for term in list(node.inputs().values()):
-                    #print "  checking terminal", term
+                    # print("  checking terminal", term)
                     deps = list(term.connections().keys())
-                    update = False
                     for d in deps:
                         if d in terms:
-                            #print "    ..input", d, "changed"
-                            update = True
+                            # print("    ..input", d, "changed")
+                            update |= True
                             term.inputChanged(d, process=False)
-                    if update:
-                        #print "  processing.."
-                        node.update()
-                        terms |= set(node.outputs().values())
+                if update:
+                    # print("  processing..")
+                    node.update()
+                    terms |= set(node.outputs().values())
                     
         finally:
             self.processing = False
@@ -402,27 +416,25 @@ class Flowchart(Node):
                 self.inputWasSet = False
             else:
                 self.sigStateChanged.emit()
-        
-        
 
     def chartGraphicsItem(self):
-        """Return the graphicsItem which displays the internals of this flowchart.
-        (graphicsItem() still returns the external-view item)"""
-        #return self._chartGraphicsItem
+        """Return the graphicsItem that displays the internal nodes and
+        connections of this flowchart.
+        
+        Note that the similar method `graphicsItem()` is inherited from Node
+        and returns the *external* graphical representation of this flowchart."""
         return self.viewBox
         
     def widget(self):
+        """Return the control widget for this flowchart.
+        
+        This widget provides GUI access to the parameters for each node and a
+        graphical representation of the flowchart.
+        """
         if self._widget is None:
             self._widget = FlowchartCtrlWidget(self)
             self.scene = self._widget.scene()
             self.viewBox = self._widget.viewBox()
-            #self._scene = QtGui.QGraphicsScene()
-            #self._widget.setScene(self._scene)
-            #self.scene.addItem(self.chartGraphicsItem())
-            
-            #ci = self.chartGraphicsItem()
-            #self.viewBox.addItem(ci)
-            #self.viewBox.autoRange()
         return self._widget
 
     def listConnections(self):
@@ -435,10 +447,11 @@ class Flowchart(Node):
         return conn
 
     def saveState(self):
+        """Return a serializable data structure representing the current state of this flowchart. 
+        """
         state = Node.saveState(self)
         state['nodes'] = []
         state['connects'] = []
-        #state['terminals'] = self.saveTerminals()
         
         for name, node in self._nodes.items():
             cls = type(node)
@@ -458,17 +471,17 @@ class Flowchart(Node):
         return state
         
     def restoreState(self, state, clear=False):
+        """Restore the state of this flowchart from a previous call to `saveState()`.
+        """
         self.blockSignals(True)
         try:
             if clear:
                 self.clear()
             Node.restoreState(self, state)
             nodes = state['nodes']
-            #nodes.sort(lambda a, b: cmp(a['pos'][0], b['pos'][0]))
             nodes.sort(key=lambda a: a['pos'][0])
             for n in nodes:
                 if n['name'] in self._nodes:
-                    #self._nodes[n['name']].graphicsItem().moveBy(*n['pos'])
                     self._nodes[n['name']].restoreState(n['state'])
                     continue
                 try:
@@ -476,7 +489,6 @@ class Flowchart(Node):
                     node.restoreState(n['state'])
                 except:
                     printExc("Error creating node %s: (continuing anyway)" % n['name'])
-                #node.graphicsItem().moveBy(*n['pos'])
                 
             self.inputNode.restoreState(state.get('inputNode', {}))
             self.outputNode.restoreState(state.get('outputNode', {}))
@@ -489,56 +501,54 @@ class Flowchart(Node):
                     print(self._nodes[n1].terminals)
                     print(self._nodes[n2].terminals)
                     printExc("Error connecting terminals %s.%s - %s.%s:" % (n1, t1, n2, t2))
-                    
                 
         finally:
             self.blockSignals(False)
             
-        self.sigChartLoaded.emit()
         self.outputChanged()
+        self.sigChartLoaded.emit()
         self.sigStateChanged.emit()
-        #self.sigOutputChanged.emit()
             
     def loadFile(self, fileName=None, startDir=None):
+        """Load a flowchart (``*.fc``) file.
+        """
         if fileName is None:
             if startDir is None:
                 startDir = self.filePath
             if startDir is None:
                 startDir = '.'
             self.fileDialog = FileDialog(None, "Load Flowchart..", startDir, "Flowchart (*.fc)")
-            #self.fileDialog.setFileMode(QtGui.QFileDialog.AnyFile)
-            #self.fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptSave) 
             self.fileDialog.show()
             self.fileDialog.fileSelected.connect(self.loadFile)
             return
             ## NOTE: was previously using a real widget for the file dialog's parent, but this caused weird mouse event bugs..
-            #fileName = QtGui.QFileDialog.getOpenFileName(None, "Load Flowchart..", startDir, "Flowchart (*.fc)")
-        fileName = unicode(fileName)
+        fileName = asUnicode(fileName)
         state = configfile.readConfigFile(fileName)
         self.restoreState(state, clear=True)
         self.viewBox.autoRange()
-        #self.emit(QtCore.SIGNAL('fileLoaded'), fileName)
         self.sigFileLoaded.emit(fileName)
-        
+
     def saveFile(self, fileName=None, startDir=None, suggestedFileName='flowchart.fc'):
+        """Save this flowchart to a .fc file
+        """
         if fileName is None:
             if startDir is None:
                 startDir = self.filePath
             if startDir is None:
                 startDir = '.'
             self.fileDialog = FileDialog(None, "Save Flowchart..", startDir, "Flowchart (*.fc)")
-            #self.fileDialog.setFileMode(QtGui.QFileDialog.AnyFile)
+            self.fileDialog.setDefaultSuffix("fc")
             self.fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptSave) 
-            #self.fileDialog.setDirectory(startDir)
             self.fileDialog.show()
             self.fileDialog.fileSelected.connect(self.saveFile)
             return
-            #fileName = QtGui.QFileDialog.getSaveFileName(None, "Save Flowchart..", startDir, "Flowchart (*.fc)")
-        fileName = unicode(fileName)
+        fileName = asUnicode(fileName)
         configfile.writeConfigFile(self.saveState(), fileName)
         self.sigFileSaved.emit(fileName)
 
     def clear(self):
+        """Remove all nodes from this flowchart except the original input/output nodes.
+        """
         for n in list(self._nodes.values()):
             if n is self.inputNode or n is self.outputNode:
                 continue
@@ -551,18 +561,15 @@ class Flowchart(Node):
         self.inputNode.clearTerminals()
         self.outputNode.clearTerminals()
 
-#class FlowchartGraphicsItem(QtGui.QGraphicsItem):
+
 class FlowchartGraphicsItem(GraphicsObject):
     
     def __init__(self, chart):
-        #print "FlowchartGraphicsItem.__init__"
-        #QtGui.QGraphicsItem.__init__(self)
         GraphicsObject.__init__(self)
         self.chart = chart ## chart is an instance of Flowchart()
         self.updateTerminals()
         
     def updateTerminals(self):
-        #print "FlowchartGraphicsItem.updateTerminals"
         self.terminals = {}
         bounds = self.boundingRect()
         inp = self.chart.inputs()
@@ -619,7 +626,10 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         self.cwWin.resize(1000,800)
         
         h = self.ui.ctrlList.header()
-        h.setResizeMode(0, h.Stretch)
+        if QT_LIB in ['PyQt4', 'PySide']:
+            h.setResizeMode(0, h.Stretch)
+        else:
+            h.setSectionResizeMode(0, h.Stretch)
         
         self.ui.ctrlList.itemChanged.connect(self.itemChanged)
         self.ui.loadBtn.clicked.connect(self.loadClicked)
@@ -656,7 +666,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         #self.setCurrentFile(newFile)
         
     def fileSaved(self, fileName):
-        self.setCurrentFile(unicode(fileName))
+        self.setCurrentFile(asUnicode(fileName))
         self.ui.saveBtn.success("Saved.")
         
     def saveClicked(self):
@@ -685,7 +695,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         #self.setCurrentFile(newFile)
             
     def setCurrentFile(self, fileName):
-        self.currentFileName = unicode(fileName)
+        self.currentFileName = asUnicode(fileName)
         if fileName is None:
             self.ui.fileNameLabel.setText("<b>[ new ]</b>")
         else:
@@ -755,6 +765,10 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         item = self.items[node]
         self.ui.ctrlList.setCurrentItem(item)
 
+    def clearSelection(self):
+        self.ui.ctrlList.selectionModel().clearSelection()
+
+
 class FlowchartWidget(dockarea.DockArea):
     """Includes the actual graphical flowchart and debugging interface"""
     def __init__(self, chart, ctrl):
@@ -825,9 +839,9 @@ class FlowchartWidget(dockarea.DockArea):
     def buildMenu(self, pos=None):
         def buildSubMenu(node, rootMenu, subMenus, pos=None):
             for section, node in node.items():
-                menu = QtGui.QMenu(section)
-                rootMenu.addMenu(menu)
-                if isinstance(node, OrderedDict): 
+                if isinstance(node, OrderedDict):
+                    menu = QtGui.QMenu(section)
+                    rootMenu.addMenu(menu)
                     buildSubMenu(node, menu, subMenus, pos=pos)
                     subMenus.append(menu)
                 else:
@@ -881,7 +895,10 @@ class FlowchartWidget(dockarea.DockArea):
             item = items[0]
             if hasattr(item, 'node') and isinstance(item.node, Node):
                 n = item.node
-                self.ctrl.select(n)
+                if n in self.ctrl.items:
+                    self.ctrl.select(n)
+                else:
+                    self.ctrl.clearSelection()
                 data = {'outputs': n.outputValues(), 'inputs': n.inputValues()}
                 self.selNameLabel.setText(n.name())
                 if hasattr(n, 'nodeName'):
@@ -929,4 +946,3 @@ class FlowchartWidget(dockarea.DockArea):
         
 class FlowchartNode(Node):
     pass
-
