@@ -6,7 +6,6 @@ from .Terminal import *
 from ..pgcollections import OrderedDict
 from ..debug import *
 import numpy as np
-from .eq import *
 
 
 def strDict(d):
@@ -223,7 +222,7 @@ class Node(QtCore.QObject):
         for t in self.inputs().values():
             nodes |= set([i.node() for i in t.inputTerminals()])
         return nodes
-        #return set([t.inputTerminals().node() for t in self.listInputs().itervalues()])
+        #return set([t.inputTerminals().node() for t in self.listInputs().values()])
         
     def __repr__(self):
         return "<Node %s @%x>" % (self.name(), id(self))
@@ -261,7 +260,7 @@ class Node(QtCore.QObject):
         for k, v in args.items():
             term = self._inputs[k]
             oldVal = term.value()
-            if not eq(oldVal, v):
+            if not fn.eq(oldVal, v):
                 changed = True
             term.setValue(v, process=False)
         if changed and '_updatesHandled_' not in args:
@@ -374,7 +373,7 @@ class Node(QtCore.QObject):
         pos = self.graphicsItem().pos()
         state = {'pos': (pos.x(), pos.y()), 'bypass': self.isBypassed()}
         termsEditable = self._allowAddInput | self._allowAddOutput
-        for term in self._inputs.values() + self._outputs.values():
+        for term in list(self._inputs.values()) + list(self._outputs.values()):
             termsEditable |= term._renamable | term._removable | term._multiable
         if termsEditable:
             state['terminals'] = self.saveTerminals()
@@ -478,7 +477,7 @@ class NodeGraphicsItem(GraphicsObject):
         #self.node.sigTerminalRenamed.connect(self.updateActionMenu)
         
     #def setZValue(self, z):
-        #for t, item in self.terminals.itervalues():
+        #for t, item in self.terminals.values():
             #item.setZValue(z+1)
         #GraphicsObject.setZValue(self, z)
         
@@ -511,31 +510,41 @@ class NodeGraphicsItem(GraphicsObject):
         
         
     def updateTerminals(self):
-        bounds = self.bounds
         self.terminals = {}
         inp = self.node.inputs()
-        dy = bounds.height() / (len(inp)+1)
-        y = dy
+        out = self.node.outputs()
+        
+        maxNode = max(len(inp), len(out))
+        titleOffset = 25
+        nodeOffset = 12
+        
+        # calculate new height
+        newHeight = titleOffset+maxNode*nodeOffset
+        
+        # if current height is not equal to new height, update
+        if not self.bounds.height() == newHeight:
+            self.bounds.setHeight(newHeight)
+            self.update()
+
+        # Populate inputs
+        y = titleOffset
         for i, t in inp.items():
             item = t.graphicsItem()
             item.setParentItem(self)
             #item.setZValue(self.zValue()+1)
-            br = self.bounds
             item.setAnchor(0, y)
             self.terminals[i] = (t, item)
-            y += dy
+            y += nodeOffset
         
-        out = self.node.outputs()
-        dy = bounds.height() / (len(out)+1)
-        y = dy
+        # Populate inputs
+        y = titleOffset
         for i, t in out.items():
             item = t.graphicsItem()
             item.setParentItem(self)
             item.setZValue(self.zValue())
-            br = self.bounds
-            item.setAnchor(bounds.width(), y)
+            item.setAnchor(self.bounds.width(), y)
             self.terminals[i] = (t, item)
-            y += dy
+            y += nodeOffset
         
         #self.buildMenu()
         
