@@ -1,12 +1,14 @@
+# -*- coding: utf-8 -*-
+import sys
 import numpy as np
 import pytest
 import pyqtgraph as pg
 from pyqtgraph.Qt import QtCore, QtTest
-from pyqtgraph.tests import assertImageApproved, mouseMove, mouseDrag, mouseClick, TransposedImageItem
-
+from pyqtgraph.tests import assertImageApproved, mouseMove, mouseDrag, mouseClick, TransposedImageItem, resizeWindow
+import pytest
 
 app = pg.mkQApp()
-
+pg.setConfigOption("mouseRateLimit", 0)
 
 def test_getArrayRegion(transpose=False):
     pr = pg.PolyLineROI([[0, 0], [27, 0], [0, 28]], closed=True)
@@ -32,7 +34,6 @@ def test_getArrayRegion(transpose=False):
         finally:
             pg.setConfigOptions(imageAxisOrder=origMode)
     
-
 def test_getArrayRegion_axisorder():
     test_getArrayRegion(transpose=True)
 
@@ -43,8 +44,7 @@ def check_getArrayRegion(roi, name, testResize=True, transpose=False):
     #win = pg.GraphicsLayoutWidget()
     win = pg.GraphicsView()
     win.show()
-    win.resize(200, 400)
-    
+    resizeWindow(win, 200, 400)
     # Don't use Qt's layouts for testing--these generate unpredictable results.
     #vb1 = win.addViewBox()
     #win.nextRow()
@@ -97,7 +97,6 @@ def check_getArrayRegion(roi, name, testResize=True, transpose=False):
     vb2.enableAutoRange(True, True)
     
     app.processEvents()
-    
     assertImageApproved(win, name+'/roi_getarrayregion', 'Simple ROI region selection.')
 
     with pytest.raises(TypeError):
@@ -135,7 +134,12 @@ def check_getArrayRegion(roi, name, testResize=True, transpose=False):
     rgn = roi.getArrayRegion(data, img1, axes=(1, 2))
     img2.setImage(rgn[0, ..., 0])
     app.processEvents()
-    assertImageApproved(win, name+'/roi_getarrayregion_inverty', 'Simple ROI region selection, view inverted.')
+    # on windows, one edge of one ROI handle is shifted slightly; letting this slide with pxCount=10
+    if pg.Qt.QT_LIB in {'PyQt4', 'PySide'}:
+        pxCount = 10
+    else:
+        pxCount=-1
+    assertImageApproved(win, name+'/roi_getarrayregion_inverty', 'Simple ROI region selection, view inverted.', pxCount=pxCount)
 
     roi.setState(initState)
     img1.resetTransform()
@@ -149,6 +153,7 @@ def check_getArrayRegion(roi, name, testResize=True, transpose=False):
     # allow the roi to be re-used
     roi.scene().removeItem(roi)
 
+    win.hide()
 
 def test_PolyLineROI():
     rois = [
@@ -159,7 +164,7 @@ def test_PolyLineROI():
     #plt = pg.plot()
     plt = pg.GraphicsView()
     plt.show()
-    plt.resize(200, 200)
+    resizeWindow(plt, 200, 200)
     vb = pg.ViewBox()
     plt.scene().addItem(vb)
     vb.resize(200, 200)
@@ -210,17 +215,26 @@ def test_PolyLineROI():
         # click segment
         mouseClick(plt, pt, QtCore.Qt.LeftButton)
         assertImageApproved(plt, 'roi/polylineroi/'+name+'_click_segment', 'Click mouse over segment.')
+
+        # drag new handle
+        mouseMove(plt, pt+pg.Point(10, -10)) # pg bug: have to move the mouse off/on again to register hover
+        mouseDrag(plt, pt, pt + pg.Point(10, -10), QtCore.Qt.LeftButton)
+        assertImageApproved(plt, 'roi/polylineroi/'+name+'_drag_new_handle', 'Drag mouse over created handle.')
         
+        # clear all points
         r.clearPoints()
         assertImageApproved(plt, 'roi/polylineroi/'+name+'_clear', 'All points cleared.')
         assert len(r.getState()['points']) == 0
         
+        # call setPoints
         r.setPoints(initState['points'])
         assertImageApproved(plt, 'roi/polylineroi/'+name+'_setpoints', 'Reset points to initial state.')
         assert len(r.getState()['points']) == 3
         
+        # call setState
         r.setState(initState)
         assertImageApproved(plt, 'roi/polylineroi/'+name+'_setstate', 'Reset ROI to initial state.')
         assert len(r.getState()['points']) == 3
-        
-    
\ No newline at end of file
+    
+    plt.hide()
+    
