@@ -1,28 +1,24 @@
 # -*- coding: utf-8 -*-
 """
-graphicsWindows.py -  Convenience classes which create a new window with PlotWidget or ImageView.
-Copyright 2010  Luke Campagnola
-Distributed under MIT/X11 license. See license.txt for more infomation.
+DEPRECATED:  The classes below are convenience classes that create a new window
+containting a single, specific widget. These classes are now unnecessary because
+it is possible to place any widget into its own window by simply calling its
+show() method.
 """
 
-from .Qt import QtCore, QtGui
+from .Qt import QtCore, QtGui, mkQApp
 from .widgets.PlotWidget import *
 from .imageview import *
 from .widgets.GraphicsLayoutWidget import GraphicsLayoutWidget
 from .widgets.GraphicsView import GraphicsView
-QAPP = None
-
-def mkQApp():
-    if QtGui.QApplication.instance() is None:
-        global QAPP
-        QAPP = QtGui.QApplication([])
 
 
 class GraphicsWindow(GraphicsLayoutWidget):
     """
-    Convenience subclass of :class:`GraphicsLayoutWidget 
-    <pyqtgraph.GraphicsLayoutWidget>`. This class is intended for use from 
-    the interactive python prompt.
+    (deprecated; use :class:`~pyqtgraph.GraphicsLayoutWidget` instead)
+    
+    Convenience subclass of :class:`~pyqtgraph.GraphicsLayoutWidget`. This class
+    is intended for use from the interactive python prompt.
     """
     def __init__(self, title=None, size=(800,600), **kargs):
         mkQApp()
@@ -34,6 +30,9 @@ class GraphicsWindow(GraphicsLayoutWidget):
         
 
 class TabWindow(QtGui.QMainWindow):
+    """
+    (deprecated)
+    """
     def __init__(self, title=None, size=(800,600)):
         mkQApp()
         QtGui.QMainWindow.__init__(self)
@@ -45,13 +44,15 @@ class TabWindow(QtGui.QMainWindow):
         self.show()
         
     def __getattr__(self, attr):
-        if hasattr(self.cw, attr):
-            return getattr(self.cw, attr)
-        else:
-            raise NameError(attr)
+        return getattr(self.cw, attr)
     
 
 class PlotWindow(PlotWidget):
+    sigClosed = QtCore.Signal(object)
+
+    """
+    (deprecated; use :class:`~pyqtgraph.PlotWidget` instead)
+    """
     def __init__(self, title=None, **kargs):
         mkQApp()
         self.win = QtGui.QMainWindow()
@@ -63,8 +64,17 @@ class PlotWindow(PlotWidget):
             self.win.setWindowTitle(title)
         self.win.show()
 
+    def closeEvent(self, event):
+        PlotWidget.closeEvent(self, event)
+        self.sigClosed.emit(self)
+
 
 class ImageWindow(ImageView):
+    sigClosed = QtCore.Signal(object)
+
+    """
+    (deprecated; use :class:`~pyqtgraph.ImageView` instead)
+    """
     def __init__(self, *args, **kargs):
         mkQApp()
         self.win = QtGui.QMainWindow()
@@ -75,9 +85,12 @@ class ImageWindow(ImageView):
         ImageView.__init__(self, self.win)
         if len(args) > 0 or len(kargs) > 0:
             self.setImage(*args, **kargs)
+        
         self.win.setCentralWidget(self)
         for m in ['resize']:
             setattr(self, m, getattr(self.win, m))
-        #for m in ['setImage', 'autoRange', 'addItem', 'removeItem', 'blackLevel', 'whiteLevel', 'imageItem']:
-            #setattr(self, m, getattr(self.cw, m))
         self.win.show()
+    
+    def closeEvent(self, event):
+        ImageView.closeEvent(self, event)
+        self.sigClosed.emit(self)
