@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import pyqtgraph as pg
 import gc, os
 import pytest
@@ -10,11 +11,16 @@ def test_isQObjectAlive():
     o2 = pg.QtCore.QObject()
     o2.setParent(o1)
     del o1
-    gc.collect()
     assert not pg.Qt.isQObjectAlive(o2)
 
-@pytest.mark.skipif(pg.Qt.USE_PYSIDE, reason='pysideuic does not appear to be '
-                                             'packaged with conda')
+@pytest.mark.skipif(pg.Qt.QT_LIB == 'PySide', reason='pysideuic does not appear to be '
+                                                     'packaged with conda')
+@pytest.mark.skipif(
+    pg.Qt.QT_LIB == "PySide2"
+    and tuple(map(int, pg.Qt.PySide2.__version__.split("."))) >= (5, 14) 
+    and tuple(map(int, pg.Qt.PySide2.__version__.split("."))) < (5, 14, 2, 2), 
+    reason="new PySide2 doesn't have loadUi functionality"
+)
 def test_loadUiType():
     path = os.path.dirname(__file__)
     formClass, baseClass = pg.Qt.loadUiType(os.path.join(path, 'uictest.ui'))
